#ifndef __FILE_H
#define __FILE_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>
#include <stdint.h>
#include <fat.h>
#include <sys/dir.h>
#include <unistd.h>
#include <smb.h>
#include "common.h"

#include "ringbuffer.h"

#define MAX_INIT_RETRIES    20
#define MAX_SHARE_LENGTH    100

typedef enum
{
	STREAM_NONE,
	STREAM_ROOT,
	STREAM_FILE,
	STREAM_HTTP,
	STREAM_ICY,
	STREAM_MAX
} StreamType;

typedef enum
{
	FILE_NONE = 0,
	FILE_NORMAL = 1,
	FILE_DIRECTORY = 2,
	FILE_SOURCE = 3
} FileType;

typedef struct
{
    char *protocol;
    DISC_INTERFACE *interface;
    uint32_t revision;
    bool failed;
    bool writeable;
} FileSystem;

typedef struct
{
    char name[MAX_SHARE_LENGTH];
    char location[MAX_SHARE_LENGTH];
    char share[MAX_SHARE_LENGTH];
    char user[MAX_SHARE_LENGTH];
    char pass[MAX_SHARE_LENGTH];
} ShareList;

class File
{
	public:
		File();
		~File();
		
        bool exists(const char *url, const char *mode);
		int open(const char *url, const char *mode);
		int read(const void *buffer, uint32_t size, uint32_t count);
		int write(const void *buffer, uint32_t size, uint32_t count);
		void close();
		
		int seek(int32_t offset, int origin);
		uint32_t tell();
		
		int findNextFile(char *file, FileType &f);
		int findNextString(char *string, int len);
		
		void getCurrentUrl(char *url);
		
		void readAhead();
		void enumerateInterfaces();
        void enumerateShares();
		
		bool isValidURL(char *url);
        
#ifdef DEBUG_MODE
        int getCurrentParameter( int param );
#endif
	private:
		int openfat(const char *url, const char *mode);
		int readInternal(const void *buffer, uint32_t amount);
		
		int seekInternal(int offset, int origin);
        
        void addInterface( const char * const protocol, DISC_INTERFACE * const interface );
        FileSystem *findFileSystem( const char * const url );
        bool hasFailed( const char * const protocol );
        void setFailed( const char * const protocol );
        bool isWriteable( const char * const protocol );
        uint32_t getRevision( const char * const protocol );
        void tryOpenFile( FileSystem *fs );
		
		StreamType fType;
		char currentUrl[MAXPATHLEN];
		
		FILE *fp;
		DIR_ITER *dir;
		
		RingBuffer rBuf;
		
		static int instance;
        
        static FileSystem *filesystems;
        static uint32_t filesystemcount;
        
        static ShareList *shares;
        static uint32_t sharecount;
		
		bool listedBack;
		uint32_t rootPos;
        
        uint32_t revision;
		
		char last_char;
        
#ifdef DEBUG_MODE
        static int readAheadAmount;
        static int readAheadMax;
#endif
};

#ifdef __cplusplus
}
#endif

#endif


