#include <string.h>
#include <malloc.h>
#include "gmesource.h"

#define PLAYBACK_RATE	48000

GmeSource::GmeSource() : SoundSource()
{
	// Set defaults
	emu = NULL;
}

GmeSource::~GmeSource()
{
	// Kill file
	close();
}

OpenStatus GmeSource::openLocation(File *sourceFile)
{
	// Try to open wav file
	_sourceFile = sourceFile;
	
	// Grab length
	sourceFile->seek(0, SEEK_END);
	uint32_t size = sourceFile->tell();
	sourceFile->seek(0, SEEK_SET);
	
	// Grab file to memory
	void *buffer = malloc(size);
	sourceFile->read(buffer, 1, size);
	
	if(gme_open_data(buffer, size, &emu, PLAYBACK_RATE))
	{
		// Free memory
		free(buffer);
		
		return OPEN_FAILURE;
	}
	
	// Free memory
	free(buffer);
	
	// Grab number of tracks
	track_count = gme_track_count(emu);
	
	// Start track 0
	track = 0;
	gme_start_track(emu, track);
	
	// Return success
	return OPEN_SUCCESS;
}

int GmeSource::renderChunk(uint8_t *buf, int &renderSamples)
{
	// Ensure no blips at end
	memset(buf, 0, renderSamples * 4);
	
	// Render out to stereo using game music emu function
	gme_play(emu, renderSamples * 2, (short int *)buf);
	
	// More data is available
	return 1;
}

void GmeSource::seekSeconds(int seconds)
{
	// No Seeking
	track = (seconds + 2) / 5;
	
	// Start new track
	gme_start_track(emu, track);
}

int GmeSource::tellSeconds()
{
	// Calculate number of seconds in
	return track * 5;
}

bool GmeSource::hasTime()
{
	// Disable seconds display
	return false;
}

int GmeSource::getLength()
{
	// Length in seconds to sound processor
	if(track_count < 2)
	{
		return -1;
	}
	else
	{
		// Fake seconds for seeking
		return (track_count - 1) * 5;
	}
}

uint32_t GmeSource::getFrequency()
{
	// Send back pre-fetched frequency of playback
	return PLAYBACK_RATE;
}

void GmeSource::close()
{
	// Close emu
	if(emu)
	{
		gme_delete( emu );
		emu = NULL;
	}
}

int GmeSource::supportsFormat(const char *ext)
{
	// Lots of formats
	if(strcmp(ext, "ay") == 0) { return 1; }
	if(strcmp(ext, "gbs") == 0) { return 1; }
	if(strcmp(ext, "gym") == 0) { return 1; }
	if(strcmp(ext, "hes") == 0) { return 1; }
	if(strcmp(ext, "kss") == 0) { return 1; }
	if(strcmp(ext, "nsf") == 0) { return 1; }
	if(strcmp(ext, "nsfe") == 0) { return 1; }
	if(strcmp(ext, "sap") == 0) { return 1; }
	if(strcmp(ext, "spc") == 0) { return 1; }
	if(strcmp(ext, "vgm") == 0) { return 1; }
	//if(strcmp(ext, "vgz") == 0) { return 1; }

	// We support no other formats, so this function always fails
	return 0;
}
