#include <string.h>
#include <malloc.h>
#include "playlist.h"

bool isNumber(char c)
{
	if(c < '0' || c > '9')
		return false;
	
	return true;
}

Playlist::Playlist()
{
	// Ensure initialized properly
	numEntries = 0;
	curEntry = 0;
	
	entries = NULL;
}

Playlist::~Playlist()
{
	close();
}

void Playlist::fillBlanks()
{
	// fills any empty descriptions with the url so it looks better	
	if(!entries || numEntries == 0) // no entries
	{
		return;
	}
	
	for(int i = 0; i < numEntries; i++)
	{
		if(!entries[i].descriptionEntry && entries[i].urlEntry)
		{
			// it's blank, copy over the url			
			entries[i].descriptionEntry = strdup(entries[i].urlEntry);
		}
	}
}

bool Playlist::loadM3U(const char *fName)
{
	int urlLength = 0;
	int dirLength = 0;
	char str[MAXPATHLEN + 1];
	char lastTitle[MAXPATHLEN + 1];
	bool isExt = false;
	
	// Enumerate backwards to find last slash
	for(int i = strlen(fName) - 1; i >= 0; i--)
	{
		if(fName[i] == '/')
		{
			// Found directory
			dirLength = i + 1;
			break;
		}
	}
	
	// Enumerate backwards to find last slash
	for(int i = 0; i < (int)strlen(fName); i++)
	{
		if(fName[i] == '/')
		{
			// Found directory
			urlLength = i + 1;
			break;
		}
	}
	
	File fp;
	fp.open(fName, "r");
	
	// Grab the header
	fp.findNextString(str, MAXPATHLEN);
	
	if(strncmp(str, "#EXTM3U", 7) == 0)
	{
		isExt = true;
	}
	
	// Set string to no length
	lastTitle[0] = 0;
	
	while(true)
	{
		if(str[0] != '#') // ignore comments
		{
			if(numEntries == 0)
			{
				// Create structure
				numEntries = 1;
				entries = (PlsType *)malloc(sizeof(PlsType));
			}
			else
			{
				// Create more space
				numEntries++;
				entries = (PlsType *)realloc(entries, sizeof(PlsType) * numEntries);
			}
			
			// Ensure nothing linked yet
			entries[numEntries - 1].descriptionEntry = NULL;
			entries[numEntries - 1].urlEntry = NULL;
			
			char *tmpStr = str;
			
			if(fp.isValidURL(tmpStr))
			{
				// Direct copy
				entries[numEntries - 1].urlEntry = strdup(tmpStr);
			}
			else
			{
				if(tmpStr[0] == '/' || tmpStr[0] == '\\') // assume its an absolute directory
				{
					if(strlen(tmpStr) > 0)
					{
						// Figure out length
						entries[numEntries - 1].urlEntry = (char *)malloc(urlLength + strlen(tmpStr) + 1);
						
						// Copy in base directory and concatenate the current string
						memcpy(entries[numEntries - 1].urlEntry, fName, urlLength);
						memcpy(entries[numEntries - 1].urlEntry + urlLength, tmpStr, strlen(tmpStr) + 1);
					}
					else
					{
						// Fake entry
						entries[numEntries - 1].urlEntry = strdup("");
					}			
				}
				else
				{
					// check first to see if we have a drive letter					
					if(((tmpStr[0] >= 'A' && tmpStr[0] <= 'Z') || (tmpStr[0] >= 'a' && tmpStr[0] <= 'z')) && tmpStr[1] == ':')
					{
						tmpStr+=2; // skip drive
					}
					
					if(tmpStr[0] == '/' || tmpStr[0] == '\\') // assume its an absolute directory
					{
						if(strlen(tmpStr) > 0)
						{
							// Figure out length
							entries[numEntries - 1].urlEntry = (char *)malloc(urlLength + strlen(tmpStr) + 1);
							
							// Copy in base directory and concatenate the current string
							memcpy(entries[numEntries - 1].urlEntry, fName, urlLength);
							memcpy(entries[numEntries - 1].urlEntry + urlLength, tmpStr, strlen(tmpStr) + 1);
						}
						else
						{
							// Fake entry
							entries[numEntries - 1].urlEntry = strdup("");
						}
					}
					else
					{
						if(strlen(tmpStr) > 0)
						{
							// Figure out length
							entries[numEntries - 1].urlEntry = (char *)malloc(dirLength + strlen(tmpStr) + 1);
							
							// Copy in base directory and concatenate the current string
							memcpy(entries[numEntries - 1].urlEntry, fName, dirLength);
							memcpy(entries[numEntries - 1].urlEntry + dirLength, tmpStr, strlen(tmpStr) + 1);
						}
						else
						{
							// Fake entry
							entries[numEntries - 1].urlEntry = strdup("");
						}
					}
				}
				
				// change windows \ to unix-friendly /				
				for(int i = 0; i < (int)strlen(entries[numEntries - 1].urlEntry); i++)
				{
					if(entries[numEntries - 1].urlEntry[i] == '\\')
					{
						entries[numEntries - 1].urlEntry[i] = '/';
					}
				}
			}
			
			if(isExt && strlen(lastTitle) > 0)
			{
				// Absolute name is direct copy
				entries[numEntries - 1].descriptionEntry = strdup(lastTitle);
				
				// Set string to no length
				lastTitle[0] = 0;
			}	
			else
			{
				if(fp.isValidURL(entries[numEntries - 1].descriptionEntry))
				{
					// Absolute name is direct copy
					entries[numEntries - 1].descriptionEntry = strdup(entries[numEntries - 1].urlEntry);
				}
				else
				{
					// Create title out of name
					for(int i = strlen(entries[numEntries - 1].urlEntry); i >= 0; i--)
					{
						if(entries[numEntries - 1].urlEntry[i] == '/')
						{
							// Copy shallow pathname
							entries[numEntries - 1].descriptionEntry = strdup(entries[numEntries - 1].urlEntry + i + 1);
							break;
						}
					}
				}
			}
		}
		else
		{
			if(isExt) // parse it for the file title
			{
				memset(lastTitle, 0, MAXPATHLEN);
				if(strncmp(str, "#EXTINF", 7) == 0)
				{	
					char *tmpStr = strchr(str, ',');
					
					if(tmpStr)
						strcpy(lastTitle, tmpStr+1);
				}
			}
		}
		
		if(fp.findNextString(str, MAXPATHLEN) < 0)
		{
			fp.close();			
			fillBlanks();
			
			return true;
		}
	}
}

bool Playlist::loadPlaylist(const char *fName)
{
	int dirLength = 0;
	int urlLength = 0;
	char str[MAXPATHLEN + 1];
	
	// Enumerate backwards to find last slash
	for(int i = strlen(fName) - 1; i >= 0; i--)
	{
		if(fName[i] == '/')
		{
			// Found directory
			dirLength = i + 1;
			break;
		}
	}
	
	// Enumerate backwards to find last slash
	for(int i = 0; i < (int)strlen(fName); i++)
	{
		if(fName[i] == '/')
		{
			// Found directory
			urlLength = i + 1;
			break;
		}
	}
	
	File fp;
	fp.open(fName, "r");
	
	fp.findNextString(str, MAXPATHLEN);
	strlwr(str);
	
	if(strncmp(str, "[playlist]", 10) != 0)
	{
		fp.close();
		return loadM3U(fName); // assume its either a misnamed m3u or a raw list
	}
	
	// its a pls format file
	while(numEntries == 0)
	{	
		if(fp.findNextString(str, MAXPATHLEN) < 0)
		{
			fp.close();
			return false;
		}
		
		// Ensure we can compare
		strlwr(str);
		
		// Did we find the number of entries list?
		if(strncmp(str, "numberofentries=", 16) == 0)
		{			
			numEntries = 0;
			
			// we got it
			for(int i = 16; i < (int)strlen(str); i++)
			{
				if(isNumber(str[i]))
				{
					numEntries *= 10;
					numEntries += (str[i] - '0');
				}
			}
			
			// Rewind
			fp.seek(0, SEEK_SET);
		}
	}
	
	// Allocate enough room
	entries = (PlsType *)malloc(sizeof(PlsType) * numEntries);
	
	// Loop through and set all to null
	for(int i = 0; i < numEntries; i++)
	{
		entries[numEntries - 1].descriptionEntry = NULL;
		entries[numEntries - 1].urlEntry = NULL;
	}
	
	// Fill information
	while(true)
	{
		if(fp.findNextString(str, MAXPATHLEN) < 0)
		{
			fp.close();			
			fillBlanks();
			
			return true;
		}
		
		// Grab the leftmost information
		char leftFour[5];
		strncpy(leftFour, str, 4);
		leftFour[4] = 0;
		strlwr(leftFour);
		
		// Figure out which field we are loading
		int whichOne = 0;		
		
		if(strcmp(leftFour, "titl") == 0) // description field
		{
			for(int i = 5; str[i] != '='; i++)
			{
				if(isNumber(str[i]))
				{
					whichOne *= 10;
					whichOne += (str[i] - '0');
				}
				
				// Bad parse?
				if(str[i] == 0)
				{
					whichOne = 0;
					break;
				}
			}
			
			// Asign if good
			if(whichOne > 0 && whichOne <= numEntries) // we are in bounds
			{
				// Copy over
				entries[whichOne - 1].descriptionEntry = strdup(strchr(str, '=') + 1);
			}
		}
		
		if(strcmp(leftFour, "file") == 0) // url field
		{
			for(int i = 4; str[i] != '='; i++)
			{
				if(isNumber(str[i]))
				{
					whichOne *= 10;
					whichOne += (str[i] - '0');
				}
				
				// Bad parse?
				if(str[i] == 0)
				{
					whichOne = 0;
					break;
				}
			}
			
			if(whichOne > 0 && whichOne <= numEntries) // we are in bounds
			{
				char *tmpStr = strchr(str, '=') + 1;
				
				if(fp.isValidURL(tmpStr))
				{
					// Direct copy
					entries[whichOne - 1].urlEntry = strdup(tmpStr);
				}
				else
				{
					if(tmpStr[0] == '/' || tmpStr[0] == '\\') // assume its an absolute directory
					{
						if(strlen(tmpStr) > 0)
						{
							// Figure out length
							entries[whichOne - 1].urlEntry = (char *)malloc(urlLength + strlen(tmpStr) + 1);
							
							// Copy in base directory and concatenate the current string
							memcpy(entries[whichOne - 1].urlEntry, fName, urlLength);
							memcpy(entries[whichOne - 1].urlEntry + urlLength, tmpStr, strlen(tmpStr) + 1);
						}
						else
						{
							// Fake entry
							entries[whichOne - 1].urlEntry = strdup("");
						}		
					}
					else
					{
						// check first to see if we have a drive letter					
						if(((tmpStr[0] >= 'A' && tmpStr[0] <= 'Z') || (tmpStr[0] >= 'a' && tmpStr[0] <= 'z')) && tmpStr[1] == ':')
						{
							tmpStr+=2; // skip drive
						}
						
						if(tmpStr[0] == '/' || tmpStr[0] == '\\') // assume its an absolute directory
						{
							if(strlen(tmpStr) > 0)
							{
								// Figure out length
								entries[whichOne - 1].urlEntry = (char *)malloc(urlLength + strlen(tmpStr) + 1);
								
								// Copy in base directory and concatenate the current string
								memcpy(entries[whichOne - 1].urlEntry, fName, urlLength);
								memcpy(entries[whichOne - 1].urlEntry + urlLength, tmpStr, strlen(tmpStr) + 1);
							}
							else
							{
								// Fake entry
								entries[whichOne - 1].urlEntry = strdup("");
							}			
						}
						else
						{
							if(strlen(tmpStr) > 0)
							{
								// Figure out length
								entries[whichOne - 1].urlEntry = (char *)malloc(dirLength + strlen(tmpStr) + 1);
								
								// Copy in base directory and concatenate the current string
								memcpy(entries[whichOne - 1].urlEntry, fName, dirLength);
								memcpy(entries[whichOne - 1].urlEntry + dirLength, tmpStr, strlen(tmpStr) + 1);
							}
							else
							{
								// Fake entry
								entries[whichOne - 1].urlEntry = strdup("");
							}
						}
					}
					
					// change windows \ to unix-friendly /				
					for(int i = 0; i < (int)strlen(entries[whichOne - 1].urlEntry); i++)
					{
						if(entries[whichOne - 1].urlEntry[i] == '\\')
						{
							entries[whichOne - 1].urlEntry[i] = '/';
						}
					}
				}
			}
		}
	}
}

int Playlist::open(const char *url)
{
	// Close old playlost
	close();
	
	// Load playlist
	loadPlaylist(url);
	
	// Rewind to ".." entry
	curEntry = -1;
	
	// Ok!
	return 0;
}

int Playlist::findNextEntry(char *fileName, char *displayName, FileType &f)
{	
	// Back element?
	if(curEntry == -1)
	{
		strcpy(fileName, "..");
		strcpy(displayName, fileName);
		
		// Make sure it sees it as a directory
		f = FILE_DIRECTORY;
		
		// Jump to first valid file
		curEntry++;
		
		return 1;
	}

	while(true)
	{
		if(!entries || !numEntries || curEntry == numEntries)
		{
			// Last one
			fileName[0] = 0;
			displayName[0] = 0;
			f = FILE_NONE;
			
			// No file found
			return 0;
		}
		
		// Grab current elements entries
		if(entries[curEntry].urlEntry)
		{
			strcpy(fileName, entries[curEntry].urlEntry);
		}
		else
		{
			fileName[0] = 0;
		}
		
		if(entries[curEntry].descriptionEntry)
		{
			strcpy(displayName, entries[curEntry].descriptionEntry);
		}
		else
		{
			displayName[0] = 0;
		}
		
		// Don't accept this if it's blank
		if(strlen(fileName) == 0)
		{
			curEntry++;
		}
		else
		{
			break;
		}
	}
	
	f = FILE_NORMAL;
	
	// Jump to next in list
	curEntry++;
	
	// Element found
	return 1;
}

void Playlist::close()
{
	if(entries)
	{
		for(int i = 0; i < numEntries; i++)
		{
			// Free individual elements
			if(entries[i].descriptionEntry) { free(entries[i].descriptionEntry); }
			if(entries[i].urlEntry) { free(entries[i].urlEntry); }
			
			entries[i].descriptionEntry = NULL;
			entries[i].urlEntry = NULL;
		}
		
		// Free container
		free(entries);
		
		entries = NULL;
	}
	
	numEntries = 0;
}
