#include <string.h>
#include <malloc.h>
#include <sidplay.h>

#include "sidsource.h"

#define PLAYBACK_RATE	48000

SidSource::SidSource() : SoundSource()
{
	// Set up emu
	buffer = NULL;
}

SidSource::~SidSource()
{
	// Kill file
	close();
}

OpenStatus SidSource::openLocation(File *sourceFile)
{
	// Try to open wav file
	_sourceFile = sourceFile;
	
	// Grab length
	sourceFile->seek(0, SEEK_END);
	uint32_t size = sourceFile->tell();
	sourceFile->seek(0, SEEK_SET);
	
	// Grab file to memory
	buffer = malloc(size);
	sourceFile->read(buffer, 1, size);
	
	c64Init();
	synth_init(PLAYBACK_RATE);
	
	LoadSIDFromMemory(buffer, &sid_load_addr, &sid_init_addr, &sid_play_addr, &track_count, &track, &play_speed, size);
	track_count++;
	
	// Turn on full volume
	sidPoke(24, 15);
	
	// Start the song initialize
	cpuJSR(sid_init_addr, track);
	
	// Ensure no rendering happens yet
	nSamplesPerCall = 882;
	nSamplesToRender = 0;
	
	// Return success
	return OPEN_SUCCESS;
}

int SidSource::renderChunk(uint8_t *buf, int &renderSamples)
{
	// Ensure no blips at end
	memset(buf, 0, renderSamples * 4);
	
	int samples_to_render = renderSamples;
	unsigned short *tBuffer = (unsigned short *)buf;
	
	int nSamplesRendered = 0;
	while (nSamplesRendered < samples_to_render)
	{
		if (nSamplesToRender == 0)
		{
			cpuJSR(sid_play_addr, 0);
			
			// Find out if cia timing is used and how many samples
			// have to be calculated for each cpujsr
			int nRefreshCIA = sidGetCIA();	
			if ((nRefreshCIA==0) || (play_speed == 0)) 
				nRefreshCIA = 20000;
			nSamplesPerCall = PLAYBACK_RATE * nRefreshCIA / 1000000;
			
			nSamplesToRender = nSamplesPerCall;
		}
		if (nSamplesRendered + nSamplesToRender > samples_to_render)
		{
			synth_render(tBuffer + nSamplesRendered, samples_to_render - nSamplesRendered);
			nSamplesToRender -= samples_to_render - nSamplesRendered;
			nSamplesRendered = samples_to_render;
		}
		else
		{
			synth_render(tBuffer + nSamplesRendered, nSamplesToRender);
			nSamplesRendered += nSamplesToRender;
			nSamplesToRender = 0;
		} 
	}
	
	// Copy in own footprint
	for(int x = renderSamples - 1; x >= 0; x--)
	{
		tBuffer[(x * 2) + 1] = tBuffer[x];
		tBuffer[(x * 2)] = tBuffer[x];
	}
	
	// More data is available
	return 1;
}

void SidSource::seekSeconds(int seconds)
{
	// No Seeking
	track = (seconds + 2) / 5;
	
	// Start the song initialize
	cpuJSR(sid_init_addr, track);
}

int SidSource::tellSeconds()
{
	// Calculate number of seconds in
	return track * 5;
}

bool SidSource::hasTime()
{
	// Disable seconds display
	return false;
}

int SidSource::getLength()
{
	// Length in seconds to sound processor
	if(track_count < 2)
	{
		return -1;
	}
	else
	{
		// Fake seconds for seeking
		return (track_count - 1) * 5;
	}
}

uint32_t SidSource::getFrequency()
{
	// Send back pre-fetched frequency of playback
	return PLAYBACK_RATE;
}

void SidSource::close()
{
	// Kill file
	if(buffer) { free(buffer); }
	
	buffer = NULL;
}

int SidSource::supportsFormat(const char *ext)
{
	// Only one format
	if(strcmp(ext, "sid") == 0) { return 1; }
	
	// We support no other formats, so this function always fails
	return 0;
}
