#ifndef __SOUND_H
#define __SOUND_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>

#include "soundsource.h"
#include "ringbuffer.h"
#include "fft.h"
#include "light.h"

typedef enum
{
	SOUND_UNLOADED,
	SOUND_STOPPED,
	SOUND_PLAYING,
	SOUND_PAUSED
} SoundState;

class Sound
{
	public:
		Sound();
		~Sound();
		
		void update();
        void updateBuffers();
		
		int loadSound(const char *url);
		void playSound();
		void stopSound();
		void pauseSound();
		void closeSound();
		
		int getSoundLength();
		int getSoundPosition();
		bool getTimeEnabled();
		void setSoundPosition(int pos);
		SoundState getSoundState();
		
		void resetNaturalFinish();
		int finishedNaturally();
		
		void getNowPlaying(char *str);
		void getFFTResult(int *result);
		
		int playsFormat(const char *url);
		
		void enableDiskLight(bool enabled);
		bool getDiskLightEnabled();
        
#ifdef DEBUG_MODE
        int getCurrentParameter( int param );
#endif
	private:
		void stopSoundInternal();
        void clearData();
		
		static int instance;
		
		static Light light;
		static bool _lenabled;
		
		static FFT fft;
		static File sourceFile;
		static SoundSource *soundSource;
		static SoundSource **availableSources;
		
		static int loaded;
		static int started;
		static int finished;
		static int paused;
		static int naturalFinish;
};

#ifdef __cplusplus
}
#endif

#endif
