#include <string.h>
#include "soundsource.h"

SoundSource::SoundSource()
{
	// Ensure no bad open causes a false success on reading
	_length = 0;
	
	// Ensure no new format
	newFormat[0] = 0;
}

SoundSource::~SoundSource()
{
	// Empty destructor
}

OpenStatus SoundSource::openLocation(File *sourceFile)
{
	// Do nothing!
	_sourceFile = sourceFile;
	
	return OPEN_SUCCESS;
}

int SoundSource::renderChunk(uint8_t *buf, int &renderSamples)
{
	// Render blank data (16bit stereo, so 2 bytes per sample, 2 channels)
	memset(buf, 0, renderSamples * 4);
	
	// Signify more data available
	return 1;
}

void SoundSource::seekSeconds(int seconds)
{
	// No seeking here
}

int SoundSource::tellSeconds()
{
	// No length
	return -1;
}

int SoundSource::getLength()
{
	// No length
	return -1;
}

uint32_t SoundSource::getFrequency()
{
	// Standard frequency
	return 48000;
}

bool SoundSource::hasTime()
{
	// Flat out enable time display
	return true;
}

void SoundSource::close()
{
	// No cleanup here
}

void SoundSource::getNowPlaying(char *output)
{
	// Grab the filename
	char tFile[MAXPATHLEN];
	_sourceFile->getCurrentUrl(tFile);
	
	// Find the filename only
	for(int i = strlen(tFile); i >= 0; i--)
	{
		if(tFile[i] == '/')
		{
			// Found it!
			strcpy(output, &tFile[i + 1]);
			
			// Exit
			return;
		}
	}
	
	// Default
	strcpy(output, languageManager.getString(LANGUAGE_NOTHING));
}

int SoundSource::supportsFormat(const char *ext)
{
	// We support no formats, so this function always fails
	return 0;
}

void SoundSource::getFormat(char *fmt)
{
	// Copy out the new format for this file
	strcpy(fmt, newFormat);
}

void SoundSource::readAhead()
{
	// Nothing for this control
}
