#ifndef __TEXTRENDER_H
#define __TEXTRENDER_H

#ifdef __cplusplus
extern "C" {
#endif

#define DEFAULT_X 5
#define DEFAULT_Y 5

#define DEFAULT_Y_CUSHION 2
#define DEFAULT_TAB_SPACE 50

#define DEFAULT_X_RIGHT_EDGE 5

#include <stdint.h>
#include <ft2build.h>
#include FT_FREETYPE_H

#include "color.h"
#include "sound.h"
#include "theme.h"

typedef struct
{
	int left;
	int top;
	int width;
	int rows;
	int advance;
	uint8_t *data;
} Buffer;

typedef struct
{
    char name[256];
    uint8_t *data;
    uint32_t length;
    int reference;
} FontCache;

class TextRender
{
	public:
		TextRender();
		~TextRender();
		
		void setFont(const char *font);
		void setColor(Color c);
		void setSize(int s);
		
		void setRenderPosition(int x, int y);
		void setBuffer(uint8_t *buf, int width, int height);
		void renderSimple(const char *out);
		void render(const char *fmt, ...);
		
		int getRenderedWidth();
		int getRenderedHeight();
	private:
		void Blit(uint8_t *bmpBuf, int runWidth, int runHeight, int left, int top);
        FT_Error __FT_New_Face( FT_Library library, char* filepathname, FT_Long face_index, FT_Face *aface );
        void closeHandle();
        int getHandle( const char *filepathname );
		
		static FT_Library library;
		static int renderCount;
		
		Theme windowManager;
		FT_Face face;
        int handle;
		
		Sound sound;
		
        uint8_t *_font;
		uint8_t *_buf;
		int _width;
		int _height;
		int _fontheight;
		Color _color;
		
		int _renderx;
		int _rendery;
		
		int _renderedwidth;
		int _renderedheight;
		
		Buffer spriteBuf[256];
};

#ifdef __cplusplus
}
#endif

#endif


