#include <string.h>
#include <malloc.h>
#include "ymsource.h"

#define PLAYBACK_RATE	48000

YmSource::YmSource() : SoundSource()
{
	// Set defaults
	pMusic = NULL;
}

YmSource::~YmSource()
{
	// Kill file
	close();
}

OpenStatus YmSource::openLocation(File *sourceFile)
{
	// Try to open wav file
	_sourceFile = sourceFile;
	
	// Grab length
	sourceFile->seek(0, SEEK_END);
	uint32_t size = sourceFile->tell();
	sourceFile->seek(0, SEEK_SET);
	
	// Grab file to memory
	void *buffer = malloc(size);
	sourceFile->read(buffer, 1, size);
	
	pMusic = ymMusicCreate();
	
	if(!pMusic)
	{
		// Failed
		free(buffer);
		
		return OPEN_FAILURE;
	}
	
	if( ymMusicLoadMemory(pMusic, buffer, size) != YMTRUE )
	{
		// Failed
		free(buffer);
		
		// Ensure no leftover emulator
		ymMusicDestroy(pMusic);
		pMusic = NULL;
		
		return OPEN_FAILURE;
	}
	
	// Loop forever
	ymMusicSetLoopMode(pMusic, YMTRUE);
	
	// Unpause the internal implementation
	ymMusicPlay(pMusic);
	
	// Return success
	return OPEN_SUCCESS;
}

int YmSource::renderChunk(uint8_t *buf, int &renderSamples)
{
	// Ensure no blips at end
	memset(buf, 0, renderSamples * 4);
	
	// Cast for easy manipulation
	unsigned short *tBuffer = (unsigned short *)buf;
	ymMusicCompute(pMusic, (ymsample *)tBuffer, renderSamples);
	
	// Copy in own footprint
	for(int x = renderSamples - 1; x >= 0; x--)
	{
		tBuffer[(x * 2) + 1] = tBuffer[x];
		tBuffer[(x * 2)] = tBuffer[x];
	}
	
	// More data is available
	return 1;
}

uint32_t YmSource::getFrequency()
{
	// Send back pre-fetched frequency of playback
	return PLAYBACK_RATE;
}

void YmSource::close()
{
	// Close emu
	if(pMusic)
	{
		ymMusicDestroy(pMusic);
		pMusic = NULL;
	}
}

int YmSource::supportsFormat(const char *ext)
{
	// Only one
	if(strcmp(ext, "ym") == 0) { return 1; }

	// We support no other formats, so this function always fails
	return 0;
}
